import { FC } from 'react';
import { Dayjs } from 'dayjs';
export declare type SchedulerProps = {
    /**
     * The scheduler name attached to hidden input
     */
    name: string;
    /**
     * The selected date, it must be a valid parseable date string
     */
    selected?: string;
    /**
     * Interval in days for day selection
     * @default 1
     */
    dateInterval?: number;
    /**
     * Interval in minutes for time selection
     * @default 30
     */
    timeInterval?: number;
    /**
     * Date format, see [dayjs docs](https://day.js.org/docs/en/display/format)
     * @default 'ddd MMM DD'
     */
    dateFormat?: string;
    /**
     * Time format, see [dayjs docs](https://day.js.org/docs/en/display/format)
     * @default 'hh:mm A'
     */
    timeFormat?: string;
    /**
     * The duration of the event in minutes, used to check availability with ical
     * @default 30
     */
    duration?: number;
    /**
     * ICal availability public URL
     */
    icalAvailability?: string;
    /**
     * Show only the date picker(without time picker) if events lasts a whole day
     * @default false
     */
    fullDay?: boolean;
};
export declare type ICalFilter = (start?: Dayjs) => boolean;
export declare const Scheduler: FC<SchedulerProps>;
